import * as i0 from '@angular/core';
import { Component, Injectable, NgModule, APP_INITIALIZER } from '@angular/core';
import * as i1 from '@angular/router';
import { BaseRouteReuseStrategy, NavigationEnd, convertToParamMap, RouterModule, RouteReuseStrategy } from '@angular/router';
import { ReplaySubject, Subscription } from 'rxjs';
import { first, filter } from 'rxjs/operators';
import { addInitListener, addContextUpdateListener, isLuigiClientInitialized, uxManager, linkManager } from '@luigi-project/client';
import { LuigiMockEngine } from '@luigi-project/testing-utilities';

class LuigiPreloadComponent {
    constructor() { }
    ngOnInit() { }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiPreloadComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "20.0.0", type: LuigiPreloadComponent, isStandalone: true, selector: "lib-client-support-angular", ngImport: i0, template: "<p luigipreload=\"luigipreload\"></p>\n" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiPreloadComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-client-support-angular', template: "<p luigipreload=\"luigipreload\"></p>\n" }]
        }], ctorParameters: () => [] });

class LuigiContextService {
}
var ILuigiContextTypes;
(function (ILuigiContextTypes) {
    ILuigiContextTypes[ILuigiContextTypes["INIT"] = 0] = "INIT";
    ILuigiContextTypes[ILuigiContextTypes["UPDATE"] = 1] = "UPDATE";
})(ILuigiContextTypes || (ILuigiContextTypes = {}));

class LuigiContextServiceImpl {
    constructor(zone) {
        this.zone = zone;
        this.subject = new ReplaySubject(1);
        this.currentContext = null;
        addInitListener(initContext => {
            this.addListener(ILuigiContextTypes.INIT, initContext);
        });
        addContextUpdateListener(updateContext => {
            this.addListener(ILuigiContextTypes.UPDATE, updateContext);
        });
    }
    contextObservable() {
        return this.subject.asObservable();
    }
    /**
     * Get latest context object retrieved from luigi core application or empty object, if not yet set.
     */
    getContext() {
        return (this.currentContext && this.currentContext.context) || {};
    }
    /**
     * Get a promise that resolves when context is set.
     */
    getContextAsync() {
        return new Promise((resolve, reject) => {
            if (this.isObject(this.getContext()) && Object.keys(this.getContext()).length > 0) {
                resolve(this.getContext());
            }
            else {
                this.contextObservable()
                    .pipe(first())
                    .subscribe(ctx => {
                    resolve(ctx.context);
                });
            }
        });
    }
    /**
     * Checks if input is an object.
     * @param objectToCheck mixed
     * @returns {boolean}
     */
    isObject(objectToCheck) {
        return !!(objectToCheck && typeof objectToCheck === 'object' && !Array.isArray(objectToCheck));
    }
    /**
     * Set current context
     */
    setContext(obj) {
        this.zone.run(() => {
            this.currentContext = obj;
            this.subject.next(obj);
        });
    }
    addListener(contextType, context) {
        this.setContext({
            contextType,
            context
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiContextServiceImpl, deps: [{ token: i0.NgZone }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiContextServiceImpl, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiContextServiceImpl, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i0.NgZone }] });

class LuigiActivatedRouteSnapshotHelper {
    // tslint:disable-next-line:variable-name
    static { this._current = null; }
    static getCurrent() {
        return LuigiActivatedRouteSnapshotHelper._current;
    }
    static setCurrent(current) {
        LuigiActivatedRouteSnapshotHelper._current = current;
    }
}

class LuigiRouteStrategy extends BaseRouteReuseStrategy {
    retrieve(route) {
        LuigiActivatedRouteSnapshotHelper.setCurrent(route);
        return super.retrieve(route);
    }
}

class LuigiAutoRoutingService {
    constructor(router, luigiContextService) {
        this.router = router;
        this.luigiContextService = luigiContextService;
        this.subscription = new Subscription();
        this.subscription.add(this.router.events.pipe(this.doFilter()).subscribe(this.doSubscription.bind(this)));
    }
    doFilter() {
        return filter((event) => {
            return !!(event instanceof NavigationEnd &&
                event.url &&
                event.url.length > 0 &&
                !(history.state && history.state.luigiInduced));
        });
    }
    /**
     * This method will be take in consideration angular route that having in data object the paramter
     * fromVirtualTreeRoot: true, here an example:
     * {path: 'demo', component: DemoComponent, data:{fromVirtualTreeRoot: true}}
     * Another option is to specify the LuigiPath: if you add in route data luigiRoute:'/xxxx/xxx';
     * in the case we will update the path in LuigiCore navigation, here an example
     * {path: 'demo', component: DemoComponent, data:{luigiRoute: '/home/demo''}}
     * If updateModalPathParam is specified, than modalPathParam will be updated upon internal navigation:
     * {path: 'demo', component: DemoComponent, data:{updateModalPathParam: true}}
     * @param event the NavigationEnd event
     */
    doSubscription(event) {
        let current = LuigiActivatedRouteSnapshotHelper.getCurrent();
        if (!current) {
            current = this.router.routerState.root.snapshot;
            while (current?.children?.length > 0) {
                // handle multiple children
                let primary = null;
                current?.children.forEach(childSnapshot => {
                    if (childSnapshot.outlet === 'primary') {
                        primary = childSnapshot;
                    }
                });
                if (primary) {
                    current = primary;
                }
                else if (current.firstChild) {
                    current = current.firstChild;
                }
                else {
                    break;
                }
            }
        }
        if (current?.data && isLuigiClientInitialized()) {
            const ux = uxManager();
            let lm = linkManager().withoutSync();
            let route;
            if (current.data.luigiRoute) {
                route = this.getResolvedLuigiRoute(current);
                if (current.data.fromContext) {
                    if (!this.luigiContextService.getContext()) {
                        console.debug('Ignoring auto navigation request, luigi context not set');
                        return;
                    }
                    if (current.data.fromContext === true) {
                        lm = lm.fromClosestContext();
                    }
                    else {
                        lm = lm.fromContext(current.data.fromContext);
                    }
                }
            }
            else if (current.data.fromVirtualTreeRoot) {
                let url = event.url;
                const truncate = current.data.fromVirtualTreeRoot.truncate;
                if (truncate) {
                    if (truncate.indexOf('*') === 0) {
                        const index = url.indexOf(truncate.substr(1));
                        url = url.substr(index + truncate.length - 1);
                    }
                    else if (url.indexOf(truncate) === 0) {
                        url = url.substr(truncate.length);
                    }
                }
                route = url;
                console.debug('Calling fromVirtualTreeRoot for url ==> ' + route);
                lm = lm.fromVirtualTreeRoot();
            }
            if (ux.isModal()) {
                if (current.data.updateModalDataPath) {
                    lm.updateModalPathInternalNavigation(route, {}, current.data.addHistoryEntry);
                }
            }
            else if (route) {
                lm.navigate(route);
            }
        }
    }
    getResolvedLuigiRoute(current) {
        let route = current.data.luigiRoute;
        const allParams = this.getAllParamsFromParents(current);
        if (!route || !allParams) {
            return route;
        }
        const pmap = convertToParamMap(allParams);
        pmap.keys.forEach(key => {
            pmap.getAll(key).forEach(param => {
                route = route?.replace(':' + key, param);
            });
        });
        return route;
    }
    getAllParamsFromParents(current) {
        let allParams = {};
        let currentToCheck = current;
        while (currentToCheck) {
            if (currentToCheck.params) {
                allParams = { ...allParams, ...currentToCheck.params };
            }
            currentToCheck = currentToCheck.parent;
        }
        return allParams;
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiAutoRoutingService, deps: [{ token: i1.Router }, { token: LuigiContextService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiAutoRoutingService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiAutoRoutingService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.Router }, { type: LuigiContextService }] });

const staticRoutes = [
    /** here an example if you want to specify that this component is a virtualTree element in Luigi Core navigation*/
    {
        path: 'luigi-client-support-preload',
        component: LuigiPreloadComponent,
        data: { fromVirtualTreeRoot: true }
    },
    /** here an example if you want to specify that this component it is a luigi component and u want to change the navigation in Luigi core*/
    {
        path: 'luigi-client-support-preload',
        component: LuigiPreloadComponent,
        data: { luigiRoute: '/home/reload' }
    },
    /** here an example if you want to reuse the component and not recreating every time you navigate to it (a singleton Component) It requires in your module to redefine  */
    {
        path: 'luigi-client-support-preload=component',
        component: LuigiPreloadComponent,
        data: { reuse: true }
    },
    /** here an example if you want to update modalPathParam on internal navigation  */
    {
        path: 'luigi-client-support-preload',
        component: LuigiPreloadComponent,
        data: { updateModalPathParam: true }
    }
];
class LuigiAngularSupportModule {
    constructor(navigation, context) { }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiAngularSupportModule, deps: [{ token: LuigiAutoRoutingService }, { token: LuigiContextService }], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.0", ngImport: i0, type: LuigiAngularSupportModule, imports: [i1.RouterModule] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiAngularSupportModule, providers: [
            {
                provide: LuigiContextService,
                useClass: LuigiContextServiceImpl
            },
            {
                provide: RouteReuseStrategy,
                useClass: LuigiRouteStrategy
            }
        ], imports: [RouterModule.forChild(staticRoutes)] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiAngularSupportModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [RouterModule.forChild(staticRoutes)],
                    providers: [
                        {
                            provide: LuigiContextService,
                            useClass: LuigiContextServiceImpl
                        },
                        {
                            provide: RouteReuseStrategy,
                            useClass: LuigiRouteStrategy
                        }
                    ]
                }]
        }], ctorParameters: () => [{ type: LuigiAutoRoutingService }, { type: LuigiContextService }] });

// @dynamic
/*
 * This class mocks Luigi Core related functionality.
 *
 * Micro Frontends that use Luigi Client would usually communicate with Luigi Core
 * back and forth. When testing Luigi Client based components, Luigi Core might
 * not be present which leads into limitations on integration/e2e testing for standalone
 * microfrontends.
 *
 * This module adds a hook to the window postMessage API by adding an event listener to the
 * global message event of the window object and mocking the callback.
 * In the normal workflow this message would picked up by Luigi Core which then sends the response back.
 */
class LuigiMockModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiMockModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "20.0.0", ngImport: i0, type: LuigiMockModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiMockModule, providers: [
            {
                provide: APP_INITIALIZER,
                useFactory: LuigiMockEngine.initPostMessageHook,
                multi: true
            }
        ] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: LuigiMockModule, decorators: [{
            type: NgModule,
            args: [{
                    providers: [
                        {
                            provide: APP_INITIALIZER,
                            useFactory: LuigiMockEngine.initPostMessageHook,
                            multi: true
                        }
                    ]
                }]
        }] });

/*
 * Public API Surface of client-support-angular
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ILuigiContextTypes, LuigiAngularSupportModule, LuigiAutoRoutingService, LuigiContextService, LuigiContextServiceImpl, LuigiMockModule, LuigiPreloadComponent, staticRoutes };
//# sourceMappingURL=luigi-project-client-support-angular.mjs.map
