import * as i0 from '@angular/core';
import { OnInit, OnDestroy, NgZone } from '@angular/core';
import * as i1 from '@angular/router';
import { Router, RouterEvent, NavigationEnd, ActivatedRouteSnapshot, Routes } from '@angular/router';
import { Context } from '@luigi-project/client';
import { Observable, OperatorFunction } from 'rxjs';

declare class LuigiPreloadComponent implements OnInit {
    constructor();
    ngOnInit(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<LuigiPreloadComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<LuigiPreloadComponent, "lib-client-support-angular", never, {}, {}, never, never, true, never>;
}

declare abstract class LuigiContextService {
    /**
     * Listen to context changes
     * Receives current value, even if the event was already dispatched earlier.
     */
    abstract contextObservable(): Observable<IContextMessage>;
    /**
     * Get latest set context object
     */
    abstract getContext(): Context;
    /**
     * Get a promise that resolves when context is set.
     */
    abstract getContextAsync(): Promise<Context>;
}
declare enum ILuigiContextTypes {
    INIT = 0,
    UPDATE = 1
}
interface IContextMessage {
    contextType: ILuigiContextTypes;
    context: Context;
}

declare class LuigiAutoRoutingService implements OnDestroy {
    private router;
    private luigiContextService;
    private subscription;
    constructor(router: Router, luigiContextService: LuigiContextService);
    doFilter(): OperatorFunction<unknown, RouterEvent>;
    /**
     * This method will be take in consideration angular route that having in data object the paramter
     * fromVirtualTreeRoot: true, here an example:
     * {path: 'demo', component: DemoComponent, data:{fromVirtualTreeRoot: true}}
     * Another option is to specify the LuigiPath: if you add in route data luigiRoute:'/xxxx/xxx';
     * in the case we will update the path in LuigiCore navigation, here an example
     * {path: 'demo', component: DemoComponent, data:{luigiRoute: '/home/demo''}}
     * If updateModalPathParam is specified, than modalPathParam will be updated upon internal navigation:
     * {path: 'demo', component: DemoComponent, data:{updateModalPathParam: true}}
     * @param event the NavigationEnd event
     */
    doSubscription(event: NavigationEnd): void;
    getResolvedLuigiRoute(current: ActivatedRouteSnapshot): string | undefined;
    getAllParamsFromParents(current: ActivatedRouteSnapshot): {
        [key: string]: string;
    } | undefined;
    ngOnDestroy(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<LuigiAutoRoutingService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<LuigiAutoRoutingService>;
}

declare const staticRoutes: Routes;
declare class LuigiAngularSupportModule {
    constructor(navigation: LuigiAutoRoutingService, context: LuigiContextService);
    static ɵfac: i0.ɵɵFactoryDeclaration<LuigiAngularSupportModule, never>;
    static ɵmod: i0.ɵɵNgModuleDeclaration<LuigiAngularSupportModule, never, [typeof i1.RouterModule], never>;
    static ɵinj: i0.ɵɵInjectorDeclaration<LuigiAngularSupportModule>;
}

declare class LuigiContextServiceImpl implements LuigiContextService {
    private zone;
    private subject;
    private currentContext;
    constructor(zone: NgZone);
    contextObservable(): Observable<IContextMessage>;
    /**
     * Get latest context object retrieved from luigi core application or empty object, if not yet set.
     */
    getContext(): Context;
    /**
     * Get a promise that resolves when context is set.
     */
    getContextAsync(): Promise<Context>;
    /**
     * Checks if input is an object.
     * @param objectToCheck mixed
     * @returns {boolean}
     */
    private isObject;
    /**
     * Set current context
     */
    protected setContext(obj: IContextMessage): void;
    addListener(contextType: ILuigiContextTypes, context: Context): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<LuigiContextServiceImpl, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<LuigiContextServiceImpl>;
}

declare class LuigiMockModule {
    static ɵfac: i0.ɵɵFactoryDeclaration<LuigiMockModule, never>;
    static ɵmod: i0.ɵɵNgModuleDeclaration<LuigiMockModule, never, never, never>;
    static ɵinj: i0.ɵɵInjectorDeclaration<LuigiMockModule>;
}

export { ILuigiContextTypes, LuigiAngularSupportModule, LuigiAutoRoutingService, LuigiContextService, LuigiContextServiceImpl, LuigiMockModule, LuigiPreloadComponent, staticRoutes };
export type { IContextMessage };
